@@ -17,17 +17,24 @@ __docformat__ = 'restructuredtext'
 
 import contextlib
 import threading
-import zope.component
 
 try:
-    import zope.security.proxy
-except ImportError:
-    SECURITY_SUPPORT = False
-else:
-    SECURITY_SUPPORT = True
+    from zope.security.proxy import removeSecurityProxy
+except ImportError: #pragma NO COVER
+    def removeSecurityProxy(x):
+        return x
+
+from zope.component.globalregistry import getGlobalSiteManager
+from zope.component.interfaces import ComponentLookupError
+from zope.component.interfaces import IComponentLookup
 
 
 class read_property(object):
+    """Descriptor for property-like computed attributes.
+
+    Unlike the standard 'property', this descriptor allows assigning a
+    value to the instance, shadowing the property getter function.
+    """
     def __init__(self, func):
         self.func = func
 
@@ -39,20 +46,19 @@ class read_property(object):
 
 class SiteInfo(threading.local):
     site = None
-    sm = zope.component.getGlobalSiteManager()
+    sm = getGlobalSiteManager()
 
+    @read_property
     def adapter_hook(self):
         adapter_hook = self.sm.adapters.adapter_hook
         self.adapter_hook = adapter_hook
         return adapter_hook
 
-    adapter_hook = read_property(adapter_hook)
-
 siteinfo = SiteInfo()
 
 def setSite(site=None):
     if site is None:
-        sm = zope.component.getGlobalSiteManager()
+        sm = getGlobalSiteManager()
     else:
 
         # We remove the security proxy because there's no way for
@@ -62,8 +68,7 @@ def setSite(site=None):
         # once site managers do less.  There's probably no good reason why
         # they can't be proxied.  Well, except maybe for performance.
 
-        if SECURITY_SUPPORT:
-            site = zope.security.proxy.removeSecurityProxy(site)
+        site = removeSecurityProxy(site)
         # The getSiteManager method is defined by IPossibleSite.
         sm = site.getSiteManager()
 
@@ -82,8 +87,10 @@ def getSite():
 def site(site):
     old_site = getSite()
     setSite(site)
-    yield
-    setSite(old_site)
+    try:
+        yield
+    finally:
+        setSite(old_site)
 
 
 def getSiteManager(context=None):
@@ -101,34 +108,35 @@ def getSiteManager(context=None):
     # We should really look look at this again though, especially
     # once site managers do less.  There's probably no good reason why
     # they can't be proxied.  Well, except maybe for performance.
-    sm = zope.component.interfaces.IComponentLookup(
-        context, zope.component.getGlobalSiteManager())
-    if SECURITY_SUPPORT:
-        sm = zope.security.proxy.removeSecurityProxy(sm)
+    sm = IComponentLookup(
+        context, getGlobalSiteManager())
+    sm = removeSecurityProxy(sm)
     return sm
 
 
 def adapter_hook(interface, object, name='', default=None):
     try:
         return siteinfo.adapter_hook(interface, object, name, default)
-    except zope.component.interfaces.ComponentLookupError:
+    except ComponentLookupError:
         return default
 
 
 def setHooks():
-    zope.component.adapter_hook.sethook(adapter_hook)
-    zope.component.getSiteManager.sethook(getSiteManager)
+    from zope.component import _api
+    _api.adapter_hook.sethook(adapter_hook)
+    _api.getSiteManager.sethook(getSiteManager)
 
 def resetHooks():
     # Reset hookable functions to original implementation.
-    zope.component.adapter_hook.reset()
-    zope.component.getSiteManager.reset()
+    from zope.component import _api
+    _api.adapter_hook.reset()
+    _api.getSiteManager.reset()
 
 # Clear the site thread global
 clearSite = setSite
 try:
     from zope.testing.cleanup import addCleanUp
-except ImportError:
+except ImportError: #pragma NO COVER
     pass
 else:
     addCleanUp(resetHooks)
