@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2002 Zope Corporation and Contributors.
+# Copyright (c) 2002 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -12,11 +12,11 @@
 #
 ##############################################################################
 """Components for testing
-
-$Id: components.py 109324 2010-02-22 22:48:31Z sidnei $
 """
-from zope.interface import Interface, Attribute, implements
-from zope.component import adapts
+from zope.interface import Interface
+from zope.interface import Attribute
+from zope.interface import implementer
+from zope.component import adapter
 
 class IAppb(Interface):
     a = Attribute('test attribute')
@@ -33,12 +33,14 @@ class IApp3(IAppb):
 
 class IContent(Interface): pass
 
+@implementer(IContent)
 class Content(object):
-    implements(IContent)
+    pass
 
+@adapter(IContent)
+@implementer(IApp)
 class Comp(object):
-    adapts(IContent)
-    implements(IApp)
+    pass
 
     def __init__(self, *args):
         # Ignore arguments passed to constructor
